C
C    Copyright (C) 2009 Anders Aspegren Søndergaard <andersas@gmail.com>
C    -------------------------------------------------------------------

C    This file is part of Wavlib.

C    Wavlib is free software: you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation, either version 3 of the License, or
C    (at your option) any later version.

C    Wavlib is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.

C    You should have received a copy of the GNU General Public License
C    along with Wavlib. If not, see <http://www.gnu.org/licenses/>.


C     Move the data segment in a wave file
C     to the end of file. Return .TRUE. on success, .FALSE. else

      LOGICAL FUNCTION MOVDAT(U)
        IMPLICIT NONE
        INTEGER U,Y
        INTEGER I, N, J, OLDDAT,FLEN,OTHLEN,OLDOTH
        INTEGER DATLEN, LEFT
        INTEGER TOWRTE
        LOGICAL GETSTR, GTCNKS
        INCLUDE "../../common/wav.f"
        CHARACTER TEMP(BUFLEN)

C       If data segment is allready last, do nothing:
        IF (DTLAST(U)) THEN
                MOVDAT = .TRUE.
                RETURN
        ENDIF

C       Initialize variables: data position, file length, data subchunk
C       length, the combined length of subchunks after data, and the
C       start of subchunks after data 
        OLDDAT = DESCR(DESDAT,U)
        FLEN = DESCR(DESLEN,U)
        DATLEN = 8 + DESCR(DESDLN,U)
        OTHLEN = FLEN - (OLDDAT + DATLEN) + 1
        OLDOTH = OLDDAT + DATLEN

C       We need to rearrange the file.
C       Throw the garbage after DATAsubchunk in a scratch file,
C       move data subchunk to end of file and copy the scratch file
C       back.

C       Get an unused file unit for the scratch file:

        CALL GTUNIT(Y)
        IF (Y .LT. 0) THEN
                MOVDAT = .FALSE.
                RETURN
        ENDIF

C       Open the scratch file:
        
        OPEN(UNIT=Y, ACCESS='DIRECT', FORM='UNFORMATTED',RECL=BUFLEN,
     +  STATUS='SCRATCH',ERR=99)

        N = 1
C       Write the subchunks after data to the scratch file:
        DO 10 I = OLDOTH, FLEN, BUFLEN
          TOWRTE = MIN(BUFLEN,FLEN-I+1)
C         Read the data from wav file
          IF (.NOT. GETSTR(TEMP,U,I,TOWRTE)) THEN
                MOVDAT = .FALSE.
                CLOSE(Y)
                RETURN
          ENDIF
C         And write it to scratch:
          WRITE(UNIT=Y,REC=N) (TEMP(J),J=1,TOWRTE)
          N = N + 1
  10    CONTINUE

C       Data saved, move the data segment OTHLEN
C       further up. I holds the position of the end of
C       the next bit to read, and N holds the position of the
C       end of the place where the data should be written.

        I = OLDOTH - 1
        N = FLEN
C       What's left to be moved (last position - first position. +1
C       because it is inclusive):
  20    LEFT = I - OLDDAT + 1
        TOWRTE = MIN(BUFLEN,LEFT)
        IF(.NOT. GETSTR(TEMP,U,I-TOWRTE+1,TOWRTE)) THEN
                WRITE(*,*) 'Wavlib: Error: could not read file.',
     +                     'Possible wav file corruption!'
                MOVDAT = .FALSE.
                CLOSE(Y)
                RETURN
        ENDIF

        CALL SETSTR(TEMP,U,N-TOWRTE+1,TOWRTE)

        N = N - BUFLEN
        I = I - BUFLEN
C       Loop if I is still greater than OLDDAT (position of old data
C       subchunk start):
        IF (I .GE. OLDDAT) GOTO 20

C       We're now done moving the data segment. Put the subchunks
C       saved in the scratch file back into the wav file:

        N = 1
        DO 30 I = OLDDAT, OLDDAT + OTHLEN - 1, BUFLEN
          TOWRTE = MIN(BUFLEN,OTHLEN - (N-1)*BUFLEN)
C         Read the data from scratch file and write it to wav file:
          READ(UNIT=Y,REC=N) (TEMP(J), J=1,TOWRTE)
          CALL SETSTR(TEMP,U,I,TOWRTE)
          N = N + 1
  30    CONTINUE

C       Subchunks have been rearranged, get the new chunk positions:
        IF (.NOT. GTCNKS(U,DESCR(DESDAT,U),DESCR(DESDLN,U),
     +  DESCR(DESFMT,U), DESCR(DESFLN,U),DTLAST(U))) THEN
C         This is one of those that really shouldn't logically happen:
          WRITE(*,*) 'Wavlib: Warning: Possible wav file corruption!'
          MOVDAT = .FALSE.
          CLOSE(Y)
          RETURN
        ENDIF

C       All done:
        MOVDAT = .TRUE.
        CLOSE(Y)
        RETURN

C       Could not open the scratch file
  99    WRITE(*,*) 'Wavlib: Warning: Could not create scratch file.'
        MOVDAT = .FALSE.
        RETURN
      END
