C
C    Copyright (C) 2009 Anders Aspegren Søndergaard <andersas@gmail.com>
C    -------------------------------------------------------------------

C    This file is part of Wavlib.

C    Wavlib is free software: you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation, either version 3 of the License, or
C    (at your option) any later version.

C    Wavlib is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.

C    You should have received a copy of the GNU General Public License
C    along with Wavlib. If not, see <http://www.gnu.org/licenses/>.


C     Create a new, empty wave file.
C     U is an unused file unit, FNAME is the filename
C     NCHNLS is the number of channels, SRATE sample rate (samples per
C     second) and BPS is BitsPerSample, which must be an integer
C     multiple of 8, and less than the integer byte length.
C
C     It leaves the file in an opened state on success
C     and returns 0 on success, -1 on error opening file
C     (see WAVERR) and a positive number indicates
C     errors writing to the file. In case of error, the file
C     is left in a closed state.

      LOGICAL FUNCTION MKWAV(U,FNAME,NCHNLS,SRATE,BPS)
        INTEGER U, NCHNLS, SRATE, BPS
        CHARACTER FNAME*(*)
        INTEGER IOS
        COMMON /ERR/ IOS
        INCLUDE "../../common/wav.f"
        CHARACTER FNAME2*(NAMLEN)
        INTEGER LOGLEN

C       Check if filename is beyond supported length, or if it has a
C       space in the end (this means the name is ambiguous)
        IF (LOGLEN(FNAME) .LT. 1) GOTO 99
        FNAME2 = FNAME
        IF (FNAME2 .NE. FNAME .OR. LOGLEN(FNAME) .NE. LEN(FNAME))
     +    GOTO 99

C       Sanity checks. We require bits to be a positive integer
C       multiple of 8, sample rate & # of channels positive:
        IF (NCHNLS .LE. 0 .OR. SRATE .LE. 0 .OR.
     +  MOD(BPS,8) .NE. 0) GOTO 99

        IOS = 0
        OPEN(UNIT=U, FILE=FNAME, STATUS='NEW', ACCESS='DIRECT',
     +  FORM='UNFORMATTED', RECL=BUFLEN, IOSTAT=IOS, ERR=99)


C       Initialize this for the setint and setstr functions:
        DESCR(DESLEN,U) = 0

C       No samples in empty file
        DESCR(DESSAM,U) = 0
        DESCR(DESCNS,U) = NCHNLS
        DESCR(DESRTE,U) = SRATE
        DESCR(DESBPS,U) = BPS

C       Make the format subchunk the first subchunk, and
C       set the format chunk length in accordance with the documentation
        DESCR(DESFMT,U) = 13
        DESCR(DESFLN,U) = 16

C       Next subchunk. The data length is 0, because we're
C       creating a new, empty wave file:
        DESCR(DESDAT,U) = DESCR(DESFMT,U) + 8 + DESCR(DESFLN,U)
        DESCR(DESDLN,U) = 0

C       We write data segment last:
        DTLAST(U) = .TRUE.
C       No change in file and subchunk length yet:
        CHGWLN(U) = .FALSE.
        CHGSAM(U) = .FALSE.

C       Write the actual file:

C       Header
        CALL SETSTR('RIFF', U, 1, 4)
        CALL SETINT(DESCR(DESDAT,U), U, 5, 4)
        CALL SETSTR('WAVE', U, 9, 4)
C       Format subchunk header:
        CALL SETSTR('fmt ', U, DESCR(DESFMT,U), 4)
        CALL SETINT(DESCR(DESFLN,U), U, DESCR(DESFMT,U)+4,4)
C       Format subchunk:
C       PCM data in audio format: 1
        CALL SETINT(1,U,DESCR(DESFMT,U)+8,2)
        CALL SETINT(NCHNLS,U,DESCR(DESFMT,U)+8+2,2)
        CALL SETINT(SRATE,U,DESCR(DESFMT,U)+8+4,4)
        CALL SETINT(SRATE*NCHNLS*BPS/8,U,DESCR(DESFMT,U)+8+8,4)
        CALL SETINT(NCHNLS*BPS/8,U,DESCR(DESFMT,U)+8+12,2)
        CALL SETINT(BPS,U,DESCR(DESFMT,U)+8+14,2)
C       Data header
        CALL SETSTR('data', U, DESCR(DESDAT,U), 4)
        CALL SETINT(0,U,DESCR(DESDAT,U)+4,4)

        MKWAV = .TRUE.
        RETURN
 99     MKWAV = .FALSE.
        RETURN
      END

