C
C    Copyright (C) 2009 Anders Aspegren Søndergaard <andersas@gmail.com>
C    -------------------------------------------------------------------

C    This file is part of Wavlib.

C    Wavlib is free software: you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation, either version 3 of the License, or
C    (at your option) any later version.

C    Wavlib is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.

C    You should have received a copy of the GNU General Public License
C    along with Wavlib. If not, see <http://www.gnu.org/licenses/>.


C    Open an existing wave file and store relevant info
C    about for later use (eg READWAV)

C    Returns 0 on success, nonzero on failure.

      INTEGER FUNCTION OPENW(U, FNAME)
        IMPLICIT NONE
        INTEGER U, FLEN, I, IOS, DAT,FORMT
        INTEGER SMPLES, CHNLS, SPRATE, BPS, DATLEN, FMTLEN
        COMMON /ERR/ IOS
        CHARACTER*(*) FNAME
        LOGICAL TEST

        INCLUDE "../../common/wav.f"

        CHARACTER FNAME2*(NAMLEN)
        LOGICAL GETINT
        LOGICAL ISRIFF, ISWAV, GTCNKS
        INTEGER MAXWAV
        INTEGER*8 FSIZE,RES
        INTEGER LOGLEN

C       Check if the filename isn't too long/short or if it ends in space:
        IF (LOGLEN(FNAME) .LT. 1) GOTO 86
        FNAME2 = FNAME
        IF (FNAME .NE. FNAME2 .OR. LOGLEN(FNAME) .NE. LEN(FNAME))
     +   GOTO 86

        OPENW = 0
 
C       The underlying wav file handling routines
C       depend on knowing the actual file size
C       of the file we're dealing with, so get that.
C       
C       If the file size is more than 4 gigabyte + 8 byte
C       (the maximum allowed wav file length allowed
C       by the 32 bit header), refuse to work on the file.
C       Also refuse to work on the file if it has more
C       bytes than we can address with a standard INTEGER type.

C       Note that RES and FSIZE is 64 bit:
        RES = FSIZE(U,FNAME) 
        IF (RES .EQ. -2) GOTO 98
        IF (RES .LT. 0) GOTO 99

C       The MAXWAV function probes the hardware and determines
C       maximum integer.
C       If an INTEGER can address more than 4 GB + 8 byte,
C       it just sets MAXWAV to 4 GB + 8 byte.

C       IF the file is any bigger than our limit, 
        IF (RES .GT. MAXWAV()) GOTO 98

C       Else, tell out file handling subsystem what the file size is:

        DESCR(DESLEN,U) = RES

        IOS = 0
        OPEN(UNIT=U, FILE=FNAME, STATUS='OLD', ACCESS='DIRECT',
     +  FORM='UNFORMATTED', RECL=BUFLEN, IOSTAT=IOS, ERR=99)

        IF (.NOT. (ISRIFF(U) .AND. ISWAV(U))) GOTO 89

C       Check if the filesize given in the RIFF header of the wave file
C       matches the real file size.
        IF (.NOT. GETINT(FLEN, U,5,4,.TRUE.)) GOTO 89 
        FLEN = FLEN + 8
        IF (DESCR(DESLEN,U) .NE. FLEN) THEN
          WRITE(*,*) 'Wavlib: Warning: Incorrect filesize given in',
     +   ' RIFF header of ', FNAME, '.'
          WRITE(*,*) 'Difference: ', DESCR(DESLEN,U) - FLEN
        ENDIF

C        File opened, it turned out to have a RIFF WAVE file header.
C        See if it also has 'fmt ' and 'data' subchunks
C        If it does, populate DESCR array and return true.
C        Otherwise close file and return false.

        IF (.NOT. GTCNKS(U, DAT, DATLEN, FORMT, FMTLEN,
     +  DTLAST(U))) GOTO 89

C       Check if this is PCM data
        IF (.NOT. GETINT(I,U,FORMT+8,2,.TRUE.)) GOTO 89
        IF (I .NE. 1) GOTO 88

C       The documentation requires the format subchunk
C       to hold at least 16 bytes excluding header:
        IF (FMTLEN .LT. 16) GOTO 89

C       Relevant subchunks found, fill out metadata
C       on our file:

        TEST = GETINT(CHNLS,U,FORMT+10,2,.TRUE.)
        TEST = TEST .AND. GETINT(SPRATE,U,FORMT+12,4,.TRUE.)
        TEST = TEST .AND. GETINT(BPS, U,FORMT+22,2,.TRUE.)
        IF (.NOT. TEST) GOTO 89

C       Only accept an integer multiple of bytes per sample
        IF (MOD(BPS,8) .NE. 0) GOTO 87

C       We want the number of samples per track,
C       and we know the byte length of the data chunk
        SMPLES = DATLEN / (CHNLS*(BPS/8))

        DESCR(DESSAM,U) = SMPLES
        DESCR(DESDAT,U) = DAT
        DESCR(DESDLN,U) = DATLEN
        DESCR(DESFMT,U) = FORMT
        DESCR(DESFLN,U) = FMTLEN
        DESCR(DESCNS,U) = CHNLS
        DESCR(DESRTE,U) = SPRATE
        DESCR(DESBPS,U) = BPS

        DO 40 I = 1,NDESCR
                IF (DESCR(I,U) .LE. 0) GOTO 89
  40    CONTINUE
 
        RETURN

C       Filename too long:
   86   OPENW = 5
        RETURN 
C       Not an integer bytes per sample
   87   OPENW = 3
        CALL CLOSEW(U)
        RETURN
C       Not a PCM file/unsupported format:
   88   OPENW = 2
        CALL CLOSEW(U)
        RETURN
C       Invalid wave file:
   89   OPENW = 1
        CALL CLOSEW(U)
        RETURN
C       File is too big:
   98   OPENW = 4
        RETURN
C       Could not open file:
  99    OPENW = -1
        RETURN
      END

