C
C    Copyright (C) 2009 Anders Aspegren Søndergaard <andersas@gmail.com>
C    -------------------------------------------------------------------

C    This file is part of Wavlib.

C    Wavlib is free software: you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation, either version 3 of the License, or
C    (at your option) any later version.

C    Wavlib is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.

C    You should have received a copy of the GNU General Public License
C    along with Wavlib. If not, see <http://www.gnu.org/licenses/>.


C     READW reads an open wave file, U. It gets NSMPLS samples
C     starting at position START and reads it into
C     SMPLE. NCHNLS should tell how many channels to read,
C     CHNLS(1:NCHNLS) what channels and SMPLE 
C     is a real array that gets filled with the requested
C     samples. Column I in SMPLE contains the channel CHANEL(I) samples
C
C     For best performance when reading .wav files sequentially,
C     loop over start position instead of channels (eg read
C     NSMPLS samples of all channels)

C     All the returned samples are in the interval [-1:1]
C     Returns the number of samples per channel read
C     (eg 0 on error), or -1 if the file is not opened
      INTEGER FUNCTION READW(U, NCHNLS, CHANEL, START, NSMPLS, SMPLE)
        IMPLICIT NONE
        INTEGER U, START, NSMPLS, NCHNLS
        INTEGER CHANEL(NCHNLS)
        REAL SMPLE(NSMPLS, NCHNLS)
        INTEGER I,N,IDX,SAM
        INCLUDE "../../common/wav.f"
        LOGICAL GETSAM
        REAL SAM2PT

C       Stop if file is not opened
        IF (DESCR(DESLEN,U) .LE. 0) THEN
                READW = -1
                RETURN
        ENDIF

        DO 10 I = 1, NCHNLS
         IF (CHANEL(I) .LE. 0 .OR. CHANEL(I) .GT. DESCR(DESCNS,U)) THEN
C               Invalid channel request
                GOTO 99
         ENDIF
  10    CONTINUE

C       If we start after the last sample
        IF (START .GT. DESCR(DESSAM,U)) GOTO 99

C       Read the actual samples. If we reach end of file
C       before SMPLE is filled up, stop and return
C       the actual samples (per channel) returned.

        READW = 0
        IDX = 1
        DO 30 I = START, MIN(START + NSMPLS - 1,DESCR(DESSAM,U))
               DO 20 N = 1,NCHNLS
                 IF (.NOT. GETSAM(U, SAM, I,CHANEL(N))) THEN
                        GOTO 40
                 ENDIF
C             Convert the found sample to a real value between -1 and 1
C             and store it in the output array:
                 SMPLE(IDX,N) = SAM2PT(SAM,DESCR(DESBPS,U))
  20           CONTINUE
               IDX = IDX + 1
               READW = READW + 1
  30    CONTINUE
  
C       READW = DESCR(DESSAM,U) - START + 1
  40  RETURN
  99  READW = 0
      RETURN
      END


