C
C    Copyright (C) 2009 Anders Aspegren Søndergaard <andersas@gmail.com>
C    -------------------------------------------------------------------

C    This file is part of Wavlib.

C    Wavlib is free software: you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation, either version 3 of the License, or
C    (at your option) any later version.

C    Wavlib is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.

C    You should have received a copy of the GNU General Public License
C    along with Wavlib. If not, see <http://www.gnu.org/licenses/>.


C     WRITEW writes an open wave file, U. It sets NSMPLS samples
C     starting at sample START in U, reading it from 
C     SMPLE. NCHNLS should tell how many channels to write,
C     CHNLS(1:NCHNLS) what channels and SMPLE 
C     is a real array filled with the requested
C     samples. Column I in SMPLE contains the channel CHANEL(I) samples
C
C     For best performance when writing .wav files sequentially,
C     loop over start position instead of channels (eg read
C     NSMPLS samples in all channels)

C     All the samples shall be in the interval [-1:1].
C     Samples outside this interval will be forcibly fit in.
C     Returns 0 on success, -1 on error.
C     On error, no samples are written.

      INTEGER FUNCTION WRITEW(U, NCHNLS, CHANEL, START, NSMPLS, SMPLE)
        IMPLICIT NONE
        INTEGER U, START, NSMPLS, NCHNLS
        INTEGER ENDSPL, ENDLEN
        INTEGER CHANEL(NCHNLS)
        REAL SMPLE(NSMPLS, NCHNLS)
        INTEGER I,N,IDX,SAM,LSTPOS 
        INTEGER MAXWAV
        INCLUDE "../../common/wav.f"
        INTEGER PT2SAM
        LOGICAL MOVDAT

C       Stop if file is not opened
        IF (DESCR(DESLEN,U) .LE. 0) THEN
                WRITEW = -1
                RETURN
        ENDIF

C       A few sanity checks
        IF (NCHNLS .LE. 0 .OR. START .LE. 0 .OR. NSMPLS .LE. 0) THEN
                WRITEW = -1
                RETURN
        ENDIF

C       Check if the channels exist
        DO 10 I = 1, NCHNLS
         IF (CHANEL(I) .LE. 0 .OR. CHANEL(I) .GT. DESCR(DESCNS,U)) THEN
C               Invalid channel request
                GOTO 99
         ENDIF
  10    CONTINUE


C       Last sample and length from first sample of file
C       to last written sample (inclusive)

        ENDSPL = START + NSMPLS - 1
        ENDLEN = ENDSPL * (DESCR(DESBPS,U)/8)*DESCR(DESCNS,U)

C       Check if the file has the data subchunk as the last
C       subchunk. If it isn't, we must rearrange the subchunks
C       to avoid writing into other subchunks.
C       Only do this, however, if we're asked to write
C       beyond the subchunks end.

        IF ((.NOT. DTLAST(U)) .AND. ENDLEN .GT. DESCR(DESDLN,U)) THEN
                IF(.NOT. MOVDAT(U)) THEN
                        WRITEW = -1
                        RETURN
                ENDIF
        ENDIF

C       Check if by writing these samples we'll be writing
C       data further out than INTEGER can represent, or 
C       that the wav file format supports.

        IF (ENDSPL .LT. 1) THEN
C               Integer overflow
                WRITEW = -1
                RETURN
        ENDIF

        LSTPOS = DESCR(DESCNS,U)*DESCR(DESBPS,U)/8
        IF (MAXWAV()/LSTPOS .LE. ENDSPL) THEN
C               LSTPOS*ENDSPL overflow:
                WRITE(*,*) MAXWAV(), LSTPOS, ENDSPL
                WRITEW = -1
                RETURN
        ENDIF
        LSTPOS = ENDSPL*LSTPOS
        LSTPOS = LSTPOS + DESCR(DESDAT,U) + 8
        IF (LSTPOS .LE. 0 .OR. LSTPOS .GT. MAXWAV()) THEN
C               The addition has overflown or we passed the
C               maximally supported integer.
                WRITEW = -1
                RETURN
        ENDIF

C       All checks done!

C       Write the actual samples.
        IDX = 1
        DO 30 I = START, ENDSPL
               DO 20 N = 1,NCHNLS
                 SAM = PT2SAM(SMPLE(IDX,N),DESCR(DESBPS,U))
                 CALL SETSAM(U, SAM, I,CHANEL(N))
  20           CONTINUE
               IDX = IDX + 1
  30    CONTINUE
  
C       If we've added more samples (ie. not just overwritten samples)
C       to the file, set CHGSAM(U):

        IF (ENDSPL .GT. DESCR(DESSAM,U)) THEN
         DESCR(DESDLN,U) = ENDLEN
         DESCR(DESSAM,U) = ENDSPL
         CHGSAM(U) = .TRUE.
        ENDIF

      WRITEW = 0
      RETURN
  99  WRITEW = -1
      RETURN
      END


