C
C    Copyright (C) 2009 Anders Aspegren Søndergaard <andersas@gmail.com>
C    -------------------------------------------------------------------

C    This file is part of Wavlib.

C    Wavlib is free software: you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation, either version 3 of the License, or
C    (at your option) any later version.

C    Wavlib is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.

C    You should have received a copy of the GNU General Public License
C    along with Wavlib. If not, see <http://www.gnu.org/licenses/>.


C     This function duplicates WRITEW, except it takes raw
C     samples instead of REAL values in the range -1.0 to 1.0.
C     Notice that 8 bit .wav files are unsigned in the range 0-255
C     and all other encodings are signed.
C     If a value is encoded with more bits than the number of bits
C     per sample, the value will be truncated to fit in.

C     Returns 0 on success, -1 on error.
C     On error, no samples are written.

      INTEGER FUNCTION WRSAM(U, NCHNLS, CHANEL, START, NSMPLS, SMPLE)
        IMPLICIT NONE
        INTEGER U, START, NSMPLS, NCHNLS
        INTEGER ENDSPL, ENDLEN
        INTEGER CHANEL(NCHNLS)
        INTEGER SMPLE(NSMPLS, NCHNLS)
        INTEGER I,N,IDX,LSTPOS 
        INTEGER MAXWAV
        INCLUDE "../../common/wav.f"
        LOGICAL MOVDAT

C       Stop if file is not opened
        IF (DESCR(DESLEN,U) .LE. 0) THEN
                WRSAM = -1
                RETURN
        ENDIF

C       A few sanity checks
        IF (NCHNLS .LE. 0 .OR. START .LE. 0 .OR. NSMPLS .LE. 0) THEN
                WRSAM = -1
                RETURN
        ENDIF

C       Check if the channels exist
        DO 10 I = 1, NCHNLS
         IF (CHANEL(I) .LE. 0 .OR. CHANEL(I) .GT. DESCR(DESCNS,U)) THEN
C               Invalid channel request
                GOTO 99
         ENDIF
  10    CONTINUE


C       Last sample and length from first sample of file
C       to last written sample (inclusive)

        ENDSPL = START + NSMPLS - 1
        ENDLEN = ENDSPL * (DESCR(DESBPS,U)/8)*DESCR(DESCNS,U)

C       Check if the file has the data subchunk as the last
C       subchunk. If it isn't, we must rearrange the subchunks
C       to avoid writing into other subchunks.
C       Only do this, however, if we're asked to write
C       beyond the subchunks end.

        IF ((.NOT. DTLAST(U)) .AND. ENDLEN .GT. DESCR(DESDLN,U)) THEN
                IF(.NOT. MOVDAT(U)) THEN
                        WRSAM = -1
                        RETURN
                ENDIF
        ENDIF

C       Check if by writing these samples we'll be writing
C       data further out than INTEGER can represent, or 
C       that the wav file format supports.

        IF (ENDSPL .LT. 1) THEN
C               Integer overflow
                WRSAM = -1
                RETURN
        ENDIF

        LSTPOS = DESCR(DESCNS,U)*DESCR(DESBPS,U)/8
        IF (MAXWAV()/LSTPOS .LE. ENDSPL) THEN
C               LSTPOS*ENDSPL overflow:
                WRITE(*,*) MAXWAV(), LSTPOS, ENDSPL
                WRSAM = -1
                RETURN
        ENDIF
        LSTPOS = ENDSPL*LSTPOS
        LSTPOS = LSTPOS + DESCR(DESDAT,U) + 8
        IF (LSTPOS .LE. 0 .OR. LSTPOS .GT. MAXWAV()) THEN
C               The addition has overflown or we passed the
C               maximally supported integer.
                WRSAM = -1
                RETURN
        ENDIF

C       All checks done!

C       Write the actual samples.
        IDX = 1
        DO 30 I = START, ENDSPL
               DO 20 N = 1,NCHNLS
                 CALL SETSAM(U, SMPLE(IDX,N), I,CHANEL(N))
  20           CONTINUE
               IDX = IDX + 1
  30    CONTINUE
  
C       If we've added more samples (ie. not just overwritten samples)
C       to the file, set CHGSAM(U):

        IF (ENDSPL .GT. DESCR(DESSAM,U)) THEN
         DESCR(DESDLN,U) = ENDLEN
         DESCR(DESSAM,U) = ENDSPL
         CHGSAM(U) = .TRUE.
        ENDIF

      WRSAM = 0
      RETURN
  99  WRSAM = -1
      RETURN
      END


