//
//    Copyright (C) 2009 Anders Aspegren Søndergaard <andersas@gmail.com>
//    -------------------------------------------------------------------

//    This file is part of Wavlib.

//    Wavlib is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    Wavlib is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

//    You should have received a copy of the GNU General Public License
//    along with Wavlib. If not, see <http://www.gnu.org/licenses/>.



//	Generate a wav file containing a simple tone

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>

int main(int argc, char **argv) {

	float hz, t, signal;

	int time, filesize, fmtsize, datasize, samplerate, ABPS;
	short int compcode, channels, blockalign, sbps;

	short int sample16;
	unsigned char sample8;

	if (argc != 5) {
		printf("Usage: %s hz sec samplerate bits (> file.wav)\n", argv[0]);
		exit(1);
	}

	sscanf(argv[1], "%f", &hz);
	sscanf(argv[2], "%i", &time);
	sscanf(argv[3], "%i", &samplerate);
	sscanf(argv[4], "%hi", &sbps);
	if (sbps != 8 && sbps != 16) {
		printf("Bits must be either 8 or 16.\n");
		exit(3);
	}
	datasize = samplerate * sbps / 8 * time;
	datasize -= datasize % ((sbps == 8) ? sizeof(sample8) : sizeof(sample16));


	/* Total header size + data */
	filesize = 12 + 24 + 8 + datasize;
	filesize = filesize - 8; /* we need the filesize minus header size */
	fmtsize = 16;
	compcode = 1; // raw PCM
	channels = 1;
	blockalign = sbps / 8 * channels;
	ABPS = samplerate * blockalign;

	
	/* Dump header */
	printf("RIFF");
	fwrite(&filesize, 1,sizeof(filesize),stdout);
	printf("WAVE");
	
	/* Dump format chunk */
	printf("fmt ");
	fwrite(&fmtsize, 1,sizeof(fmtsize),stdout);
	fwrite(&compcode, 1,sizeof(compcode),stdout);
	fwrite(&channels, 1,sizeof(channels),stdout);
	fwrite(&samplerate, 1,sizeof(samplerate),stdout);
	fwrite(&ABPS, 1,sizeof(ABPS),stdout);
	fwrite(&blockalign, 1,sizeof(blockalign),stdout);
	fwrite(&sbps, 1,sizeof(sbps),stdout);

	/* Generate the data chunk */

	printf("data");
	fwrite(&datasize, 1,sizeof(datasize),stdout);

	t = 0.0;

//	while (t < (float) time) {
	while (datasize > 0) {
		if (sbps == 8) {
			sample8 = (sin(2*M_PI*hz * t) + 1) / 2 * 255;
			fwrite(&sample8, 1,sizeof(sample8),stdout);
			datasize -= sizeof(sample8);
		} else {
			sample16 = sin(2*M_PI*hz * t) * 32767;
			fwrite(&sample16, 1,sizeof(sample16),stdout);
			datasize -= sizeof(sample16);
		}
	
		t = t + 1.0/(float)samplerate;
	}
	
return 0;
}
 
